package api

import (
	"errors"
	"fmt"
	"oilseal-train/db/model"
	"oilseal-train/db/repo"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func Test(r *GinRouter) {
	r.POSTJWT("/test/create", TestAdd)
	r.GETJWT("/test/list", TestList)
	r.POSTJWT("/test/update", TestEdit)
	r.POSTJWT("/test/delete/:id", TestDelete)
}

func TestAdd(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	err := IsAdmin(apictx)
	if err != nil {
		return nil, err
	}
	var form model.Test
	err = c.ShouldBindJSON(&form)
	if err != nil {
		fmt.Println(err)
		return nil, errors.New("参数错误")
	}
	if len(form.BankId) != 12 {
		return nil, errors.New("题库Id不正确")
	}
	if form.Question == "" {
		return nil, errors.New("题目不能为空")
	}

	if len(form.Answer) < 1 {
		return nil, errors.New("答案不能为空")
	}
	if form.State == 0 {
		form.State = 1

	}
	form.CreateTime = time.Now()
	form.UpdateTime = time.Now()
	return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionTest, &form)
}

func TestList(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	// err := IsAdmin(apictx)
	// if err != nil {
	// 	return nil, err
	// }
	page, size, query := UtilQueryPageSize(c)

	if _question, ok := query["question"]; ok {
		query["question"] = bson.M{"$regex": _question.(string)}
	}
	if _bankId, ok := query["bankId"]; ok {
		query["bankId"], _ = primitive.ObjectIDFromHex(_bankId.(string))
	}
	result, err := repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: repo.CollectionTest,
		Page:        page,
		Size:        size,
		Query:       query,
	})
	if len(result.List) == 0 {
		return result, err
	}

	// 获取bank信息
	ctx := apictx.CreateRepoCtx()
	for _, list := range result.List {
		bankId := list["bankId"].(primitive.ObjectID)
		bank := model.Bank{}
		repo.RepoSeachDoc(ctx, &repo.DocSearchOptions{
			CollectName: repo.CollectionBank,
			Query:       repo.Map{"_id": bankId},
		}, &bank)
		list["bank"] = bank
	}

	return result, err
}

func TestEdit(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	// 操作用户为admin
	err := IsAdmin(apictx)
	if err != nil {
		return nil, err
	}
	var form model.Test
	err = c.ShouldBindJSON(&form)
	if err != nil {
		return nil, errors.New("参数错误")
	}
	if len(form.Id) != 12 {
		return nil, errors.New("id不正确")
	}

	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionTest, form.Id.Hex(), &form)
}

// 删除用户
func TestDelete(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	// 操作用户为admin
	err := IsAdmin(apictx)
	if err != nil {
		return nil, err
	}
	_id := c.Param("id")
	_, err = primitive.ObjectIDFromHex(_id)
	if err != nil {
		return nil, errors.New("id错误")
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionTest, _id)
}
